-----------------------------------------------------------------------------------------------
-- Client Lua Script for ImpaleAssist
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- ImpaleAssist Module Definition
-----------------------------------------------------------------------------------------------
local ImpaleAssist = {} 
local Version = 1
local sin = math.sin
local cos =  math.cos
local rad = math.rad

 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function ImpaleAssist:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function ImpaleAssist:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- ImpaleAssist OnLoad
-----------------------------------------------------------------------------------------------
function ImpaleAssist:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("ImpaleAssist.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- ImpaleAssist OnDocLoaded
-----------------------------------------------------------------------------------------------
function ImpaleAssist:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.Overlay = Apollo.LoadForm(self.xmlDoc, "ImpaleOverlay", "InWorldHudStratum", self)
		if self.Overlay == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    self.Overlay:Show(true, true)
    	Apollo.RegisterEventHandler("NextFrame", "OnNextFrame", self)
		Apollo.RegisterTimerHandler("AbilityBook_CacheTimer",	"DelayedAbilityBookCheck", self)
		Apollo.RegisterEventHandler("AbilityBookChange", "OnAbilityBookChange", self)
		self.HasImpale = false
		
		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)


		-- Do additional Addon initialization here
	end
end

-----------------------------------------------------------------------------------------------
-- ImpaleAssist Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here


-----------------------------------------------------------------------------------------------
-- ImpaleAssistForm Functions
-----------------------------------------------------------------------------------------------
-- when the OK button is clicked
function ImpaleAssist:OnOK()
	self.wndMain:Close() -- hide the window
end

-- when the Cancel button is clicked
function ImpaleAssist:OnCancel()
	self.wndMain:Close() -- hide the window
end


-----------------------------------------------------------------------------------------------
-- ImpaleAssist Instance
-----------------------------------------------------------------------------------------------
local ImpaleAssistInst = ImpaleAssist:new()
ImpaleAssistInst:Init()

function ImpaleAssist:OnNextFrame()
	local unit = GameLib.GetPlayerUnit()
	if unit == nil or unit:GetClassId() ~= GameLib.CodeEnumClass.Stalker then return end
	
	if self.HasImpale then
		self:ImpaleTelegraph(unit)
	end

end


function ImpaleAssist:ImpaleTelegraph(unit)
	self.Overlay:DestroyAllPixies()
	if not unit:IsInCombat() then return end

	local target = unit:GetTarget()
	local length = 0
	local nResourceCurrent = unit:GetResource(3)
	if nResourceCurrent  < 30 then return end
	local uface = unit:GetFacing()
	local uangle = math.atan2(uface.x, uface.z)
	local tPos = unit:GetPosition()
	local nOffsetDegree = uangle+math.rad(180)
	local color = "ffff0000"
	
	if target then
		local xPos = target:GetPosition()
		local tVec = Vector3.New(xPos.x, xPos.y, xPos.z)
		local uVec = Vector3.New(tPos.x, tPos.y, tPos.z)
		length	= (uVec - tVec):Length()
	end
	
	local originPositionVector= { x = tPos.x+0*math.sin(nOffsetDegree) , y = tPos.y , z = tPos.z+0*math.cos(nOffsetDegree) }
	local dist = 7
	local firstVector= Vector3.New(originPositionVector.x+dist*sin(uangle), originPositionVector.y, originPositionVector.z+dist*cos(uangle))
	local startPoint = GameLib.WorldLocToScreenPoint(firstVector)

	local rotation_point = uangle + rad(17.5)
	local currentVectorA = Vector3.New(originPositionVector.x+dist*sin(rotation_point), originPositionVector.y, originPositionVector.z+dist*cos(rotation_point))
	local endPointA = GameLib.WorldLocToScreenPoint(currentVectorA)
	self.Overlay:AddPixie( { bLine = true, fWidth = 2,  cr = color, loc = { fPoints = {0,0,0,0}, nOffsets = { startPoint.x, startPoint.y, endPointA.x, endPointA.y } }    } )

	local rotation_point = uangle + rad(-17.5)
	local currentVectorAR = Vector3.New(originPositionVector.x+dist*sin(rotation_point), originPositionVector.y, originPositionVector.z+dist*cos(rotation_point))
	local endPointAR = GameLib.WorldLocToScreenPoint(currentVectorAR)
	self.Overlay:AddPixie( { bLine = true, fWidth = 2,  cr = color, loc = { fPoints = {0,0,0,0}, nOffsets = { startPoint.x, startPoint.y, endPointAR.x, endPointAR.y } }    } )

	local dist = -2.1
	local rotation_point = uangle + rad(-64.5)
	local currentVectorB = Vector3.New(currentVectorA.x+dist*sin(rotation_point), currentVectorA.y, currentVectorA.z+dist*cos(rotation_point))
	local endPointAe = GameLib.WorldLocToScreenPoint(currentVectorB)
	self.Overlay:AddPixie( { bLine = true, fWidth = 2,  cr = color, loc = { fPoints = {0,0,0,0}, nOffsets = { endPointAe.x, endPointAe.y, endPointA.x, endPointA.y } }    } )

	
	local rotation_point = uangle + rad(64.5)
	local currentVectorBR = Vector3.New(currentVectorAR.x+dist*sin(rotation_point), currentVectorAR.y, currentVectorAR.z+dist*cos(rotation_point))
	local endPointAeR = GameLib.WorldLocToScreenPoint(currentVectorBR)
	self.Overlay:AddPixie( { bLine = true, fWidth = 2,  cr = color, loc = { fPoints = {0,0,0,0}, nOffsets = { endPointAeR.x, endPointAeR.y, endPointAR.x, endPointAR.y } }    } )

	local dist = -6
	local rotation_point = uangle + rad(35)
	local currentVectorG = Vector3.New(currentVectorB.x+dist*sin(rotation_point), currentVectorB.y, currentVectorB.z+dist*cos(rotation_point))
	local endPointC = GameLib.WorldLocToScreenPoint(currentVectorG)
	self.Overlay:AddPixie( { bLine = true, fWidth = 2,  cr = color, loc = { fPoints = {0,0,0,0}, nOffsets = { endPointAe.x, endPointAe.y, endPointC.x, endPointC.y } }    } )

	
	local rotation_point = uangle + rad(-35)
	local currentVectorGR = Vector3.New(currentVectorBR.x+dist*sin(rotation_point), currentVectorBR.y, currentVectorBR.z+dist*cos(rotation_point))
	local endPointCR = GameLib.WorldLocToScreenPoint(currentVectorGR)
	self.Overlay:AddPixie( { bLine = true, fWidth = 2,  cr = color, loc = { fPoints = {0,0,0,0}, nOffsets = { endPointAeR.x, endPointAeR.y, endPointCR.x, endPointCR.y } }    } )

end


function ImpaleAssist:OnAbilityBookChange()
	self.HasImpale = false
    self.abilityBookTimer = Apollo.CreateTimer("AbilityBook_CacheTimer", 3, false)
end


function ImpaleAssist:DelayedAbilityBookCheck()
if GameLib.GetPlayerUnit() == nil then return end

	if GameLib.GetPlayerUnit():GetClassId() == 5 then
		self.HasImpale = self:AbilityBookCheck()
	end
end


function ImpaleAssist:AbilityBookCheck()
    local tCurrLAS = ActionSetLib.GetCurrentActionSet()
    if tCurrLAS then
        for nIndex, nAbilityId in ipairs(tCurrLAS) do
            if nAbilityId == 23161 then -- Impale
--				Print("found")
				return true
            end
        end
    end
	return false
end
